import math
import random
def generate_text_file(filename, content="Hello, this is a generated file!"):
    with open(filename, "w") as file:
        file.write(content)
    print(f"File '{filename}' has been created successfully.")

'''
given a string and the index of the test, generates the input file
'''
def generate_test(s, ind):
    folder = 'tests/'
    filename = folder + "monkey." + str(ind) + ".in"

    pB = random.randint(2, 1000000006)
    pA = random.randint(1, pB-1)
    gd = math.gcd(pA, pB)
    pA //= gd
    pB //= gd
    content = str(len(s))+'\n'
    content += str(pA) + ' ' + str(pB) + '\n'
    content += s
    generate_text_file(filename, content)

'''
generate tests for the first subtask, tests 1-9
'''
def generate_subtask_all_equal():
    arr = [1, 2, 3, 4, 5, 3000, 322, 2839, 73]
    for ind, n in enumerate(arr):
        s = ''
        addition = str(random.randint(0, 1))
        for i in range(n):
            s += addition
        generate_test(s, ind+1)

'''
generates tests for the second subtask, tests 10-19
'''
def generate_subtask_first_different():
    arr = [1, 2, 3, 4, 5, 2999, 83, 239, 1982, 2173]
    for i, n in enumerate(arr):
        s = ''
        addition = str(random.randint(0, 1))
        for ij in range(n):
            s += addition
        opposite = '0'
        if opposite == addition:
            opposite = '1'
        s = opposite + s
        generate_test(s, i+10)

'''
Get 0 with probability p and 1 with probability 1-p
'''
def get_num(p):
    return 0 if random.random() < p else 1

'''
returns a random string of size sz
'''
def generate_random_string(sz, p=0.5):
    s = ''
    for i in range(sz):
        s += str(get_num(p))
    return s

'''
generates a string repeating pattern; use n>=441 here
'''
def generate_test_sqrt(n, ind):
    cnt = int(math.sqrt(n))+random.randint(-20,20)
    s = generate_random_string(cnt)
    ans = ''
    for i in range(n):
        ans += s[i%cnt]
    
    generate_test(ans, ind)



'''
Generates a string with no repeating 0s, with probability p of generating a 0
'''
def generate_no_repeat_0(l, p):
    strings = ['01','1']
    ans = ''
    while len(ans)<l-1:
        ans += strings[get_num(p)]
    return ans

'''
Generates a string with kmp change probability equal to p
'''
def generate_long_kmp(l, p):
    s = ''
    s += str(random.randint(0, 1))
    arr=[0]
    for i in range(1,l):
        if get_num(p) == 0:
            s += s[arr[-1]]
            arr.append(arr[-1]+1)
        else:
            s += ('0' if s[arr[-1]] == '1' else '1')
            curind = arr[-1]
            while curind>0 and s[curind]!=s[i]:
                curind = arr[curind-1]
            if s[curind]!=s[i]:
                arr.append(0)
            else:
                arr.append(curind+1)
    return s
'''
generates tests for the subtask O(N^3)
'''
def generate_subtask_n_cubed():
    ind = 27
    generate_test('1'*200, ind)
    generate_test('01101110'*25, ind+1)
    generate_test('01'*100, ind+2)
    generate_test(generate_random_string(200), ind+3)
    generate_test(generate_random_string(30), ind+4)
    generate_test(generate_random_string(10), ind+5)
    generate_test(generate_random_string(19), ind+6)
    generate_test(generate_random_string(3)*61, ind+7)
    generate_test(generate_random_string(6)*30, ind+8)
    generate_test(generate_random_string(20)*10, ind+9)
    return

'''
If you can't guess by the name... O(N^2) btw
'''
def generate_subtask_no_repeating_zeroes():
    ind = 20
    generate_test(generate_no_repeat_0(2988, 0.01),ind)
    generate_test(generate_no_repeat_0(2988, 0.99),ind+1)
    generate_test(generate_no_repeat_0(2988, 0.5),ind+2)
    generate_test(generate_no_repeat_0(3000, 0.001),ind+3)
    generate_test(generate_no_repeat_0(2573, 0.7),ind+4)
    generate_test(generate_no_repeat_0(3000, 0.999),ind+5)
    generate_test(generate_no_repeat_0(2732, 0.2),ind+6)


'''
generates tests for the subtask O(N^2)
'''
def generate_subtask_n_squared():
    ind=37
    generate_test_sqrt(3000, ind)
    generate_test_sqrt(1500, ind+1)
    generate_test('01'*1500, ind+2)
    generate_test(generate_random_string(3000), ind+3)
    generate_test(generate_random_string(30), ind+4)
    generate_test(generate_random_string(19), ind+5)
    generate_test(generate_random_string(5)*590, ind+6)
    generate_test(generate_random_string(19)*150, ind+7)
    generate_test(generate_random_string(6)*500, ind+8)
    generate_test(generate_random_string(20)*150, ind+9)
    generate_test(generate_random_string(3000,0.01), ind+10)
    generate_test(generate_random_string(2999,0.1), ind+11)
    generate_test(generate_random_string(2977,0.99), ind+12)
    generate_test(generate_long_kmp(2900,0.1),ind+13)
    generate_test(generate_long_kmp(2900,0.01),ind+14)
    generate_test(generate_long_kmp(2900,0.9),ind+15)
    generate_test(generate_long_kmp(2900,0.99),ind+16)

'''
    generates tests for the subtask O(NlogN)
'''
def generate_subtask_n_logn():
    ind=54
    generate_test_sqrt(500000, ind)
    generate_test_sqrt(499999, ind+1)
    generate_test_sqrt(499998, ind+2)
    generate_test('01101110'*62000, ind+3)
    generate_test('10'*250000, ind+4)
    generate_test(generate_random_string(20)*25000, ind+5)
    generate_test(generate_random_string(12)*41600, ind+6)
    generate_test(generate_random_string(17)*29400, ind+7)
    generate_test(generate_random_string(5)*100000, ind+8)
    generate_test(generate_random_string(500000), ind+9)
    generate_test(generate_random_string(499999, 0.001), ind+10)
    generate_test(generate_random_string(500000, 0.1), ind+11)
    generate_test(generate_random_string(488888,0.7), ind+12)
    generate_test(generate_random_string(500000,0.99), ind+13)
    generate_test(generate_long_kmp(499996,0.1),ind+14)
    generate_test(generate_long_kmp(499996,0.01),ind+15)
    generate_test(generate_long_kmp(499996,0.9),ind+16)
    generate_test(generate_long_kmp(499996,0.99),ind+17)
    generate_test(generate_long_kmp(499996,0.997),ind+18)
    generate_test(generate_long_kmp(499996,0.999),ind+19)
    generate_test(generate_long_kmp(499996,0.001),ind+20)
    generate_test(generate_long_kmp(499996,0.995),ind+21)
    generate_test(generate_long_kmp(499996,0.9995),ind+22)
    generate_test(generate_long_kmp(499996,0.997),ind+23)
    generate_test(generate_random_string(71)*7000, ind+24)
    generate_test(generate_random_string(499999, 0.002), ind+25)
    generate_test(generate_random_string(499999, 0.003), ind+26)
    generate_test(generate_random_string(499999, 0.0007), ind+27)


'''
    generates tests for the subtask O(N)
'''
def generate_subtask_n():
    ind=82
    generate_test_sqrt(10000000, ind)
    generate_test_sqrt(9999999, ind+1)
    generate_test_sqrt(9999998, ind+2)
    generate_test('01101110'*1200000, ind+3)
    generate_test('10'*5000000, ind+4)
    generate_test(generate_random_string(3)*3330000, ind+5)
    generate_test(generate_random_string(36)*277000, ind+6)
    generate_test(generate_random_string(17)*588000, ind+7)
    generate_test(generate_random_string(9999999, 0.01), ind+8)
    generate_test(generate_random_string(10000000, 0.1), ind+9)
    generate_test(generate_random_string(9999999, 0.9), ind+10)
    generate_test(generate_random_string(10000000, 0.999), ind+11)
    generate_test(generate_long_kmp(9999999,0.1),ind+12)
    generate_test(generate_long_kmp(9999999,0.01),ind+13)
    generate_test(generate_long_kmp(9999999,0.9),ind+14)
    generate_test(generate_long_kmp(9999999,0.99),ind+15)
    generate_test(generate_long_kmp(9999999,0.999),ind+16)
    generate_test(generate_long_kmp(9999999,0.001),ind+17)
    generate_test(generate_long_kmp(9999999,0.9995),ind+18)
    generate_test(generate_long_kmp(9999999,0.996),ind+19)
    generate_test(generate_long_kmp(9999999,0.994),ind+20)

if __name__ == "__main__":
    generate_subtask_all_equal()
    generate_subtask_first_different()
    generate_subtask_n_cubed()
    generate_subtask_no_repeating_zeroes()
    generate_subtask_n_squared()
    generate_subtask_n_logn()
    generate_subtask_n()